# Required modules ###
require ifcdaq
require singlemotion
require mrfsample
require pev
require EMU,jfdenis
require ecat2db,0.3.2

### MACROS ###
# PMAC common macros3       ) 
epicsEnvSet("PMACPREFIX", "PMAC")
epicsEnvSet("PMACPORT",   "pmacLV")
epicsEnvSet("MOTOR_PORT", "pmac1" )
epicsEnvSet("NB_AXES", "1" )

# Motor Macros
epicsEnvSet("EGU",       "mm"   )
epicsEnvSet("DIR",       "Pos"  )
epicsEnvSet("MRES",      "0.000249515")
epicsEnvSet("PREC",      "4"    )
epicsEnvSet("DHLM",      "4"   )
epicsEnvSet("DLLM",      "-200"  )
epicsEnvSet("VELO",      "1"    )
epicsEnvSet("VBAS",      "0.1"  )
epicsEnvSet("VMAX",      "2"    )
epicsEnvSet("ACCL",      "0.1"  )
epicsEnvSet("BDST",      "0"    )
epicsEnvSet("INIT",      ""     )
epicsEnvSet("OFF",       "0"    )

epicsEnvSet("MOTOR_NAME1",  "MTR1")
epicsEnvSet("AXIS_NO1",     "1")

# Motor Status macros
epicsEnvSet("SCAN",         "1 second")
epicsEnvSet("OVERHEAT1",    "MAJOR"   )
epicsEnvSet("SWITCH_OFF1",  "MINOR"   )
epicsEnvSet("OVERHEAT2",    "0"       )
epicsEnvSet("SWITCH_OFF2",  "0"       )
epicsEnvSet("MOTOR_ERROR1", "0"       )
epicsEnvSet("MOTOR_ERROR2", "0"       )
epicsEnvSet("MOTOR_ERROR3", "0"       )
epicsEnvSet("MOTOR_ERROR4", "0"       )

# EVG Macros 
epicsEnvSet("SYS"             "SYS0")
epicsEnvSet("EVG"             "EVG0")
epicsEnvSet("EVG_VMESLOT"     "7")

# EVR Macros
epicsEnvSet("EVR_PCIDOMAIN",    "0000")
epicsEnvSet("EVR_PCIBUS",       "05")
epicsEnvSet("EVR_PCIDEVICE",    "00")
epicsEnvSet("EVR_PCIFUNCTION",  "0")
epicsEnvSet("EVR",              "EVR0")
epicsEnvSet("EVR_PUL0_EVENT",   "14")
epicsEnvSet("EVR_PUL0_DELAY",   "$(EVR_EV14_OUT0_DELAY=0)")
epicsEnvSet("EVR_PUL0_WIDTH",   "$(EVR_EV14_OUT0_WIDTH=1000)")

#DAQ macros
epicsEnvSet("DAQPREFIX",  "SCANNINGDAQ"  )
epicsEnvSet("DAQBUFSIZE", "1024"         )

#Scanning macros
epicsEnvSet("SCANPREFIX", "SCAN"     )
epicsEnvSet("SCAN_SLIT", "MTR"          )
epicsEnvSet("SCAN_EF", "PS"              )
epicsEnvSet("SCAN_POINTS_EF", "1000"     )
epicsEnvSet("SCAN_POINTS_SLIT", "100"    )

epicsEnvSet(EPICS_CA_MAX_ARRAY_BYTES, 400000000)

# Connection to PMAC and setup for $(NUM_MOTORS) motors
pmacAsynIPConfigure($(PMACPORT), "10.2.139.237:1025")
pmacCreateController($(MOTOR_PORT), $(PMACPORT), 0, $(NB_AXES), 50, 300)
pmacCreateAxis($(MOTOR_PORT), $(AXIS_NO1))

# Initialize EVG
mrmEvgSetupVME($(EVG), $(EVG_VMESLOT), 0x100000, 1, 0x01)

# Initialize EVR
mrmEvrSetupPCI($(EVR), $(EVR_PCIDOMAIN), $(EVR_PCIBUS), $(EVR_PCIDEVICE), $(EVR_PCIFUNCTION))

# Initialize daq
ndsCreateDevice(ifcdaq, $(DAQPREFIX), card=0, fmc=2)

### LOAD RECORDS ###

#Load EVG records
dbLoadRecords("evg-vme-230.db", "DEVICE=$(EVG), SYS=$(SYS)")

#Load EVR records
dbLoadRecords("evr-pmc-230.db", "DEVICE=$(EVR), SYS=$(SYS)")
dbLoadRecords("evr-softEvent.template", "DEVICE=$(EVR), SYS=$(SYS), EVT=$(EVR_PUL0_EVENT), CODE=$(EVR_PUL0_EVENT)")
dbLoadRecords("evr-pulserMap.template", "DEVICE=$(EVR), SYS=$(SYS), EVT=$(EVR_PUL0_EVENT), PID=0, F=Trig, ID=0")

## Load ifcdaq instance
dbLoadRecords(IFCDAQAIChannelGroup.template, "PREFIX=$(DAQPREFIX),CH_GRP_ID=AI")
dbLoadRecords(IFCDAQAIChannel.template,      "PREFIX=$(DAQPREFIX),CH_GRP_ID=AI,CH_ID=CH0, NELM=$(DAQBUFSIZE)")
dbLoadRecords(IFCDAQAIChannel.template,      "PREFIX=$(DAQPREFIX),CH_GRP_ID=AI,CH_ID=CH1, NELM=$(DAQBUFSIZE)")
dbLoadRecords(IFCDAQAIChannel.template,      "PREFIX=$(DAQPREFIX),CH_GRP_ID=AI,CH_ID=CH2, NELM=$(DAQBUFSIZE)")

#
# Load Single motions records
#

#
# Motors
#

dbLoadRecords("motor.template", "PREFIX=$(PMACPREFIX),MOTOR_NAME=$(MOTOR_NAME1),MOTOR_PORT=$(MOTOR_PORT),AXIS_NO=$(AXIS_NO1),EGU=$(EGU),DIR=$(DIR),MRES=$(MRES),PREC=$(PREC),DHLM=$(DHLM),DLLM=$(DLLM),VELO=$(VELO),VBAS=$(VBAS),VMAX=$(VMAX),ACCL=$(ACCL),BDST=$(BDST),INIT=$(INIT),OFF=$(OFF)")

#
# Motor status
#

dbLoadRecords("motorStatus.template", "PREFIX=$(PMACPREFIX),MOTOR_NAME=$(MOTOR_NAME1),ASYN_PORT=$(PMACPORT),AXIS_NO=$(AXIS_NO1),SCAN=$(SCAN),EGU=$(EGU),PREC=$(PREC),OVERHEAT1=$(OVERHEAT1),SWITCH_OFF1=$(SWITCH_OFF1),OVERHEAT2=$(OVERHEAT2),SWITCH_OFF2=$(SWITCH_OFF2),MOTOR_ERROR1=$(MOTOR_ERROR1),MOTOR_ERROR2=$(MOTOR_ERROR2),MOTOR_ERROR3=$(MOTOR_ERROR3),MOTOR_ERROR4=$(MOTOR_ERROR4)")

#
# Homing procedure records
#

dbLoadRecords("motorHoming.template", "PREFIX=$(PMACPREFIX),MOTOR_NAME=$(MOTOR_NAME1),ASYN_PORT=$(PMACPORT),AXIS_NO=$(AXIS_NO1),PREC=$(PREC),EGU=$(EGU)")

dbLoadRecords("EMU.db", "DEVICE=$(SCANPREFIX), SCAN_SLIT=$(SCAN_SLIT), SCAN_EF=$(SCAN_EF),MTRREC=$(PMACPREFIX):$(MOTOR_NAME1), MAX_POINTS_EF=$(SCAN_POINTS_EF), MAX_POINTS_SLIT=$(SCAN_POINTS_SLIT), DAQNDSPREFIX=$(DAQPREFIX), DAQBUFSIZE=$(DAQBUFSIZE), DET1WF=AI-CH0-DATA, DET2WF=AI-CH1-DATA, DET3WF=AI-CH2-DATA, TR_TL=$(SYS)-$(EVR):Pul0-Ena-Sel, TR_TLTSEL=$(SYS)-$(EVR):Event-$(EVR_PUL0_EVENT)-SP.TIME, PSU_SP=LNS-ISRC-010:PBI-EM:HVhor-SP, PSU_RBV=LNS-ISRC-010:PBI-EM:HVhor-SP, PSU2_SP=LNS-ISRC-010:PBI-EM:HVver-SP, PSU2_RBV=LNS-ISRC-010:PBI-EM:HVver-SP")

#
# Init ETHERCAT module
ecat2configure(0,500,1,1)


#############################################################################
######################### iocInit ###########################################
#############################################################################
iocInit


seq motorHoming "PREFIX=$(PMACPREFIX),MOTOR_NAME=$(MOTOR_NAME1)"

#### Simulation of a current with ES4134 from Beckoff
seq RampCurrent "PREFIX_PS=LNS-ISRC-010:PBI-EM,CONS_PS2=HVhor-SP,DEVICE=SCAN,SCAN_SLIT=$(SCAN_SLIT)"


# SET INTERFACE FROM IFCDAQ TO Allison scanner
# Forward link wf record to Detector guard
dbpf $(DAQPREFIX):AI-CH0-DATA.FLNK $(SCANPREFIX):DET1-GUARD
dbpf $(DAQPREFIX):AI-CH1-DATA.FLNK $(SCANPREFIX):DET2-GUARD
dbpf $(DAQPREFIX):AI-CH2-DATA.FLNK $(SCANPREFIX):DET3-GUARD
# Forward link from state record to Detector trigger
dbpf $(DAQPREFIX):AI-STAT-RB.FLNK $(SCANPREFIX):DAQTRG-DAQSTGUARD


# Forward link number of samples rbv to compress N record, so that detetor know number of elements to average over.
dbpf $(DAQPREFIX):AI-SMNM-RB.FLNK "$(SCANPREFIX):DET1-COMPRESSN"

### Setup IFCDAQ PVs ###
# Set the DAQ to on state
#dbpf $(DAQPREFIX):AI-STAT "ON"
# Set trigger repeat to 1
dbpf $(DAQPREFIX):AI-TRGR 1
# Set trigger source to external GPIO
dbpf $(DAQPREFIX):AI-TRGS "EXT-GPIO"
sleep(1)
dbpf $(DAQPREFIX):AI-STAT ON
sleep(3)
dbpf $(DAQPREFIX):AI-STAT RUNNING
sleep(1)
dbpf $(DAQPREFIX):AI-STAT RUNNING


# Set tsel on WF record to event record in EVR
dbpf $(DAQPREFIX):AI-CH0-DATA.TSEL $(SYS)-$(EVR):Event-$(EVR_PUL0_EVENT)-SP.TIME
dbpf $(DAQPREFIX):AI-CH1-DATA.TSEL $(SYS)-$(EVR):Event-$(EVR_PUL0_EVENT)-SP.TIME
dbpf $(DAQPREFIX):AI-CH2-DATA.TSEL $(SYS)-$(EVR):Event-$(EVR_PUL0_EVENT)-SP.TIME

########## TIMING GENERATOR: setup softEvent to trigg acquisistion on the  DTAcQ ######
## Internal trigger from Mxc1 - TrgSrc0
dbpf $(SYS)-$(EVG):TrigEvt0-EvtCode-SP 	 $(EVR_PUL0_EVENT)
dbpf $(SYS)-$(EVG):Mxc1-Frequency-SP	 1
dbpf $(SYS)-$(EVG):Mxc1-TrigSrc0-SP 	 "Set"

dbpf $(SYS)-$(EVG):ResetMxc-Cmd		 1
dbpf $(SYS)-$(EVG):SoftEvt-Enable-Sel    1

### Setup EVR
# setup OUT 0
# set the pulser 0 to trigger on output  0
# and map the pulser to trigger on event $(EVR_PUL0_EVENT)
dbpf $(SYS)-$(EVR):FrontOut0-Src-SP      0
dbpf $(SYS)-$(EVR):Pul0-Evt-Trig0-SP 	 $(EVR_PUL0_EVENT)
dbpf $(SYS)-$(EVR):Pul0-Width-SP         20000
dbpf $(SYS)-$(EVR):Pul0-Delay-SP         0
dbpf $(SYS)-$(EVR):FrontOut0-Ena-SP      1

#dbpf $(SYS)-$(EVR):Time-I.TSE -2

### Setup reasonable values for sscan record ###
# Initialize Motor positioners
dbpf $(SCANPREFIX):$(SCAN_SLIT).P4SP -10
dbpf $(SCANPREFIX):$(SCAN_SLIT).P4EP -20
dbpf $(SCANPREFIX):$(SCAN_SLIT).P2SP 1
dbpf $(SCANPREFIX):$(SCAN_SLIT).P2EP 1
dbpf $(SCANPREFIX):$(SCAN_SLIT).NPTS 11

# Initialize Power supply positioners
dbpf SCAN:PROC-AngleMax-SP 80
dbpf SCAN:PROC-AngleMin-SP -80
dbpf $(SCANPREFIX):$(SCAN_EF).NPTS 11
dbpf $(SCANPREFIX):$(SCAN_EF).PDLY 0.5



dbpf SCAN:PROC-SIMU 1

